﻿<?php

		/**
		 * class diskwriter	
		 * This class provides dal with disk writing functions.
		 * It makes sure that the files it's writing to are properly locked.
		 *
		 * It can make use of shared memory if you have much diskwriters active. So everything gets delivered
		 * to one shared memory piece and gets written to disk from there by a global diskwriter.
		 *
		 * @author		Patrick van Zweden 
		 * @version		0.0.1
		 * @package extensions
		 * @subpackage debugger
		 */
	class diskwriter {

		/**
		 * tmpfile, generated
		 */
		var $tmpFilename;


		/**
		 * Constructor of the diskwriter class
		 */
		function diskwriter()
		{
			$this->init();
		}

		function init()
		{
			include_once "debugger/config.php";
			
			$this->configuration = new debugConfiguration;
			$this->tmpFilename = md5(uniqid(rand()));
			$this->tmpFilename = $this->configuration->tmpFilePath . $this->tmpFilename;
			$this->logfile = $this->configuration->globalFilePath . $this->configuration->globalFilename;
		}

		/**
		 * This functions writes the message to the disk.
		 *
		 * scope  : public
		 * input  : string
		 * returns: boolean
		 **/
		function writeMessage($message)
		{
			$fp = fopen($this->tmpFilename, "a");

			/**
			 * While we can't get a lock on the file, keep trying
			 * for a specified amount of times.
			 * After the maximum of tries has been reached abort
			 * with an error.
			 * This prevents DAL to deadlock.
			 */
			while (!flock($fp, LOCK_EX)) {
				$lockcounter++;
				sleep($this->configuration->waitTime);

				if ($lockcounter > $this->configuration->tries) {
					fclose($fp);
					return false;
				}
			}

			/**
			 * We should have a lock on the file by now
			 */
			fwrite($fp, $message . "\n");

			/**
			 * Release the lock on the file
			 */
			flock($fp, LOCK_UN);
			fclose($fp);
		}

		/**
		 * Destructor of the class diskwriter
		 */
		function onDestroy()
		{
			$this->appendMainfile();
		}

		/**
		 * Write the temp file to the main log
		 *
		 * @access private
		 * @return boolean
		 */
		function appendMainfile()
		{
			$dest = fopen($this->logfile, "a");
			$src  = fopen($this->tmpFilename, "r");

			/**
			 * While we can't get a lock on the file, keep trying
			 * for a specified amount of times.
			 * After the maximum of tries has been reached abort
			 * with an error.
			 * This prevents DAL to deadlock.
			 */
			while (!flock($dest, LOCK_EX)) {
				$lockcounter++;
				sleep($this->waitTime);

				if ($lockcounter > $this->tries) {
					fclose($dest);
					return false;
				}
			}

			fwrite($dest, "\n--- $this->tmpFilename ---\n");
			$log = fread($src, filesize($this->tmpFilename));
			fwrite($dest, $log);
			fwrite($dest, "--- $this->tmpFilename ---\n");


			/**
			 * Release the lock on the file
			 */
			flock($dest, LOCK_UN);
			fclose($dest);
			fclose($src);

			unlink($this->tmpFilename);
		}

	}

